/*
Binaires

Copyright (C) 2002 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

import java.awt.*;

import simu.*;

/**
 * Algorithme de simulation pour SimuApplet.
 * Extinction systme binaire
 */
public class Binaires extends SimuApplet {
    
    static final int n = 200;
    static final int imgdimy = 200;
    static final int imgdimx = 400;
    double rorb2;
    double r1, temp1, l1, r2, temp2, l2, inc, rorb1;
    double tps;
    int itps;
    Color couleur1, couleur2;
    Affichage afflive;
    Image img = null;
    Graphics g;
    
    public Binaires() {
        // paramtres d'entre
        ParamIn t1 = new ParamIn();
        t1.type = "nombre";
        t1.label = "temp1";
        t1.titre = "temprature corps 1";
        t1.unit = "K";
        t1.acquisition = "champ";
        t1.defaut = "10000";
        addParamIn(t1);
        
        ParamIn r1 = new ParamIn();
        r1.type = "nombre";
        r1.label = "r1";
        r1.titre = "rayon corps 1";
        r1.unit = "m";
        r1.acquisition = "champ";
        r1.defaut = "2E9";
        addParamIn(r1);
        
        ParamIn t2 = new ParamIn();
        t2.type = "nombre";
        t2.label = "temp2";
        t2.titre = "temprature corps 2";
        t2.unit = "K";
        t2.acquisition = "champ";
        t2.defaut = "6000";
        addParamIn(t2);
        
        ParamIn r2 = new ParamIn();
        r2.type = "nombre";
        r2.label = "r2";
        r2.titre = "rayon corps 2";
        r2.unit = "m";
        r2.acquisition = "champ";
        r2.defaut = "7E8";
        addParamIn(r2);
        
        ParamIn dist = new ParamIn();
        dist.type = "nombre";
        dist.label = "dist";
        dist.titre = "distance";
        dist.unit = "m";
        dist.acquisition = "champ";
        dist.defaut = "1E10";
        addParamIn(dist);
        
        ParamIn inc = new ParamIn();
        inc.type = "nombre";
        inc.label = "inc";
        inc.titre = "inclinaison systme";
        inc.unit = "";
        inc.acquisition = "champ";
        inc.defaut = "85";
        addParamIn(inc);
        
        // paramtres de sortie
        
        ParamOut pphase = new ParamOut();
        pphase.type = "nombre";
        pphase.label = "phase";
        pphase.titre = "phase";
        pphase.unit = "tours";
        addParamOut(pphase);
        
        ParamOut plum = new ParamOut();
        plum.type = "nombre";
        plum.label = "lum";
        plum.titre = "luminosit";
        plum.unit = "%";
        addParamOut(plum);
        
        ParamOut pimage = new ParamOut();
        pimage.type = "image";
        pimage.label = "image";
        addParamOut(pimage);
        
        // options d'affichage
        
        afflive = new Affichage();
        afflive.type = "plotlive";
        afflive.titre = "Graphe extinction";
        String[][] parcourbe = {{"phase", "lum"}};
        afflive.params = parcourbe;
        addAffichage(afflive);
        
        Affichage afftableau = new Affichage();
        afftableau.type = "tableau";
        afftableau.titre = "Tableau extinction";
        afftableau.params = parcourbe;
        addAffichage(afftableau);
        
        Affichage affimage = new Affichage();
        affimage.type = "image";
        affimage.titre = "Figure binaire";
        String[][] parimage = {{"image"}};
        affimage.params = parimage;
        affimage.imgdimx = imgdimx;
        affimage.imgdimy = imgdimy;
        addAffichage(affimage);
        
        
        Affichage affmulti = new Affichage();
        affmulti.type = "multiplot";
        affmulti.titre = "Figure+Graphe";
        affmulti.sousAff = new Affichage[2];
        affmulti.sousAff[0] = affimage;
        affmulti.sousAff[1] = afflive;
        affmulti.ordreSousAff = "y";
        
        addAffichage(affmulti);
    }
    
    public void initCalculLive(ListeValeurs in) throws SimuException {
        temp1 = in.lireDouble("temp1");
        r1 = in.lireDouble("r1");
        temp2 = in.lireDouble("temp2");
        r2 = in.lireDouble("r2");
        double dist = in.lireDouble("dist");
        if (dist < r1 + r2)
            throw new SimuException("la distance est infrieure  la somme des rayons");
        inc = Math.abs(90-in.lireDouble("inc"))*Math.PI/180;
        l1 = Math.pow(r1, 2) * Math.pow(temp1, 4);
        l2 = Math.pow(r2, 2) * Math.pow(temp2, 4);
        itps = 0;
        tps = 0;
        couleur1 = couleurEtoile(temp1);
        couleur2 = couleurEtoile(temp2);
        afflive.xdebut = "0";
        afflive.xfin = "1";
        double lumin = l1 - Math.pow(r2, 2) * Math.pow(temp1, 4) + l2;
        afflive.ydebut = ""+(100*lumin/(l1 + l2));
        afflive.yfin = "100";
        // rapport de masses bidon (on suppose la densit gale entre les toiles !)
        rorb2 = dist / (Math.pow(r2, 3) / Math.pow(r1, 3) + 1);
        rorb1 = dist - rorb2;
        img = createImage(imgdimx, imgdimy);
        g = img.getGraphics();
    }
    
    public ListeValeurs calculLive() throws SimuException {
        ListeValeurs out = new ListeValeurs();
        out.ajouterDouble("phase", tps/(2*Math.PI));
        //double lum = luminosite();
        //if (Double.isNaN(lum))
        //    return(null);
        out.ajouterDouble("lum", luminosite());
        out.ajouter("image", dessinImage());
        if (itps == 0)
            afflive.effacer = true;
        else
            afflive.effacer = false;
        itps++;
        if (itps == n)
            itps = 0;
        tps = (itps * 2 * Math.PI) / n;
        return out;
    }
    
    public ListeValeurs calcul(ListeValeurs in) throws SimuException {
        initCalculLive(in);
        ListeValeurs out = new ListeValeurs();
        for (int i=0; i<n; i++) {
            tps = (i*2*Math.PI)/n;
            out.ajouterDouble("phase", tps/(2*Math.PI));
            out.ajouterDouble("lum", luminosite());
        }
        return out;
    }
    
    // le temps t va de 0  2*pi et correspond  la position du corps 2 autour du corps 1
    // les toiles sont considres comme des disques de luminosit uniforme (!)
    protected double luminosite() {
        double x1 = rorb1 * Math.cos(tps + Math.PI);
        double y1 = rorb1 * Math.sin(inc) * Math.sin(tps + Math.PI);
        double x2 = rorb2 * Math.cos(tps);
        double y2 = rorb2 * Math.sin(inc) * Math.sin(tps);
        double d = Math.sqrt((x2-x1)*(x2-x1) + (y2-y1)*(y2-y1));
        double facteur = 100 / (l1 + l2);
        
        if (d > r1 + r2) {
            //System.out.println("d > r1 + r2  "+tps+" " + (l1 + l2));
            return(facteur*(l1 + l2));
        }
        
        if (d + r2 < r1) {
            //System.out.println("d + r2 < r1");
            if (tps < Math.PI)
                return(facteur * l1);
            else
                return(facteur * (l1 - Math.pow(r2, 2) * Math.pow(temp1, 4) + l2));
        }
        
        double p = (r1 + r2 + d) / 2;
        double alos = Math.sqrt(p * (p-r1) * (p-r2) * (p-d)) * 2;
        double angc1 = 2 * Math.acos((-r2*r2 + r1*r1 + d*d)/(2*r1*d));
        double ac1 = angc1 * r1*r1 / 2;
        double angc2 = 2 * Math.acos((-r1*r1 + r2*r2 + d*d)/(2*r2*d));
        double ac2 = angc2 * r2*r2 / 2;
        double s = ac1 + ac2 - alos;
        //System.out.println("r1="+r1+" r2="+r2+" d="+d+" pangc2="+((-r1*r1 + r2*r2 + d*d)/(2*r2*d)));
        //System.out.println("p="+p+" alos="+alos+" angc1="+angc1+" ac1="+ac1+" angc2="+angc2+" ac2="+ac2+" s="+s);
        
        double lum;
        if (tps < Math.PI)
            lum = l1 + l2 - (s / Math.PI) * Math.pow(temp2, 4);
        else
            lum = l1 + l2 - (s / Math.PI) * Math.pow(temp1, 4);
        lum = facteur * lum;
        //System.out.println(tps+" " + lum);
        return(lum);
    }
    
    protected Image dessinImage() {
        double x1 = rorb1 * Math.cos(tps + Math.PI);
        double y1 = rorb1 * Math.sin(inc) * Math.sin(tps + Math.PI);
        double x2 = rorb2 * Math.cos(tps);
        double y2 = rorb2 * Math.sin(inc) * Math.sin(tps);
        double rapport = imgdimx / (2 * (rorb2 + r2*2));
        
        //Image img = createImage(imgdimx, imgdimy);
        //Graphics g = img.getGraphics();
        g.setColor(Color.black);
        g.fillRect(0, 0, imgdimx-1, imgdimy-1);
        
        int ir1 = (int)Math.round(r1*rapport);
        int id1 = ir1 * 2;
        int ix1 = (int)Math.round(imgdimx/2 + x1*rapport) - ir1;
        int iy1 = (int)Math.round(imgdimy/2 - y1*rapport) - ir1;
        int ir2 = (int)Math.round(r2*rapport);
        int id2 = ir2 * 2;
        int ix2 = (int)Math.round(imgdimx/2 + x2*rapport) - ir2;
        int iy2 = (int)Math.round(imgdimy/2 - y2*rapport) - ir2;
        //System.out.println(ix1+" "+iy1+" "+ir1+" "+ix2+" "+iy2+" "+ir2);
        
        if (tps < Math.PI) {
            g.setColor(couleur2);
            g.fillOval(ix2, iy2, id2, id2);
        }
        
        g.setColor(couleur1);
        g.fillOval(ix1, iy1, id1, id1);
        
        if (tps >= Math.PI) {
            g.setColor(couleur2);
            g.fillOval(ix2, iy2, id2, id2);
        }
        //g.dispose();
        return(img);
    }
    
    // tentative de reprsentation des couleurs apparentes
    protected static Color couleurEtoile(double temperature) {
        int r,v,b;
        if (temperature < 3000) {
            r = 255;
            v = 0;
            b = 0;
        } else if (temperature < 6000) {
            r = 255;
            v = (int)(255*(temperature-3000)/3000);
            b = 0;
        } else if (temperature < 10000) {
            b = (int)(255*(temperature-6000)/4000);
            r = 255;
            v = 255;
        } else if (temperature < 50000) {
            b = 255;
            r = 255 - (int)(255*(temperature-10000)/40000);
            v = 255 - (int)(255*(temperature-10000)/40000);
        } else {
            r = 0;
            v = 0;
            b = 255;
        }
        return(new Color(r, v, b));
    }
}

