/*
Binaires

Copyright (C) 2012 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformément aux dispositions de la Licence Publique Générale GNU, telle que publiée par la Free Software Foundation ; version 2 de la licence, ou encore (à votre choix) toute version ultérieure.

Ce programme est distribué dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de détail, voir la Licence Publique Générale GNU .

Vous devez avoir reçu un exemplaire de la Licence Publique Générale GNU en même temps que ce programme ; si ce n'est pas le cas, écrivez à la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

/**
 * Algorithme de simulation pour SimuApplet.
 * Extinction système binaire
 */

function isCanvasSupported() {
    var elem = document.createElement('canvas');
    return !!(elem.getContext && elem.getContext('2d'));
}

function versionApplet() {
    var applet = document.createElement('applet');
    var width, height; // espace disponible pour l'applet
    if (typeof(window.innerWidth) == 'number') {
        // autre que IE
        width = window.innerWidth;
        height = window.innerHeight;
    } else if ( document.documentElement && (document.documentElement.clientWidth || document.documentElement.clientHeight) ) {
        // IE 6+ en mode 'standards compliant'
        width = document.documentElement.clientWidth;
        height = document.documentElement.clientHeight;
    } else {
        width = '100%';
        height = '100%';
    }
    applet.setAttribute('width', width);
    applet.setAttribute('height', height);
    applet.setAttribute('archive', 'Binaires.jar,SimuLab.jar');
    applet.setAttribute('code', 'Binaires');

    document.body.appendChild(applet);
}

if (isCanvasSupported()) {
    new Binaires();
} else {
    // on ne peut pas utiliser excanvas ou flashcanvas pour manipuler des bitmaps offscreen
    // donc on utilise la version sous forme d'applet en alternative pour les vieux navigateurs
    if (window.addEventListener)
        window.addEventListener('load', function() { versionApplet(); }, false);
    else if (window.attachEvent)
        window.attachEvent('onload', function() { versionApplet(); });
}

function Binaires() {
    this.applet = new SimuApplet(this);
    this.applet.appletDoc('binaires_doc.xml');
    this.applet.setDelaiIterations(100);
    
    this.n = 200;
    this.imgdimy = 200;
    this.imgdimx = 400;
    this.rorb2 = 0.;
    this.r1 = 0.;
    this.temp1 = 0.;
    this.l1 = 0.;
    this.r2 = 0.;
    this.temp2 = 0.;
    this.l2 = 0.;
    this.inc = 0.;
    this.rorb1 = 0.;
    this.tps = 0.;
    this.itps = 0;
    this.couleur1 = '';
    this.couleur2 = '';
    this.afflive = null;
    this.canvas = null;
    this.ctx = null;
}

Binaires.prototype.initCalculLive = function(lin) {
    var dist, lumin;
    
    this.temp1 = lin.lireDouble('temp1');
    this.r1 = lin.lireDouble('r1');
    this.temp2 = lin.lireDouble('temp2');
    this.r2 = lin.lireDouble('r2');
    dist = lin.lireDouble('dist');
    if (dist < this.r1 + this.r2)
        throw new SimuException("la distance est inférieure à la somme des rayons");
    this.inc = Math.abs(90 - lin.lireDouble('inc')) * Math.PI/180;
    this.l1 = Math.pow(this.r1, 2) * Math.pow(this.temp1, 4);
    this.l2 = Math.pow(this.r2, 2) * Math.pow(this.temp2, 4);
    this.itps = 0;
    this.tps = 0;
    this.couleur1 = this.couleurEtoile(this.temp1);
    this.couleur2 = this.couleurEtoile(this.temp2);
    this.afflive = this.applet.getAffichageChoisi();
    if (this.afflive.type == 'multiplot')
        this.afflive = this.afflive.sousAff[1];
    this.afflive.xdebut = '0';
    this.afflive.xfin = '1';
    lumin = this.l1 - Math.pow(this.r2, 2) * Math.pow(this.temp1, 4) + this.l2;
    this.afflive.ydebut = '' + Math.floor(100 * lumin / (this.l1 + this.l2));
    this.afflive.yfin = '101';
    // rapport de masses bidon (on suppose la densité égale entre les étoiles !)
    this.rorb2 = dist / (Math.pow(this.r2, 3) / Math.pow(this.r1, 3) + 1);
    this.rorb1 = dist - this.rorb2;
    this.canvas = document.createElement('canvas');
    this.canvas.setAttribute('width', this.imgdimx);
    this.canvas.setAttribute('height', this.imgdimy);
    this.ctx = this.canvas.getContext('2d');
}

Binaires.prototype.calculLive = function() {
    var out;
    
    out = new ListeValeurs();
    out.ajouterDouble('phase', this.tps/(2*Math.PI));
    out.ajouterDouble('lum', this.luminosite());
    out.ajouter('image', this.dessinImage());
    if (this.itps == 0)
        this.afflive.effacer = true;
    else
        this.afflive.effacer = false;
    this.itps++;
    if (this.itps == this.n)
        this.itps = 0;
    this.tps = (this.itps * 2 * Math.PI) / this.n;
    return out;
}


Binaires.prototype.calcul = function(lin) {
    var out, i;
    
    this.initCalculLive(lin);
    out = new ListeValeurs();
    for (i=0; i<this.n; i++) {
        this.tps = (i * 2*Math.PI) / this.n;
        out.ajouterDouble('phase', this.tps / (2*Math.PI));
        out.ajouterDouble('lum', this.luminosite());
    }
    return out;
}

// le temps t va de 0 à 2*pi et correspond à la position du corps 2 autour du corps 1
// les étoiles sont considérées comme des disques de luminosité uniforme (!)
Binaires.prototype.luminosite = function() {
    var x1, y1, x2, y2, d, facteur, p, alos, angc1, ac1, angc2, ac2, s, lum;
    
    x1 = this.rorb1 * Math.cos(this.tps + Math.PI);
    y1 = this.rorb1 * Math.sin(this.inc) * Math.sin(this.tps + Math.PI);
    x2 = this.rorb2 * Math.cos(this.tps);
    y2 = this.rorb2 * Math.sin(this.inc) * Math.sin(this.tps);
    d = Math.sqrt((x2-x1)*(x2-x1) + (y2-y1)*(y2-y1));
    facteur = 100 / (this.l1 + this.l2);
    
    if (d > this.r1 + this.r2) {
        return(facteur * (this.l1 + this.l2));
    }
    
    if (d + this.r2 < this.r1) {
        if (this.tps < Math.PI)
            return(facteur * this.l1);
        else
            return(facteur * (this.l1 - Math.pow(this.r2, 2) * Math.pow(this.temp1, 4) + this.l2));
    }
    
    p = (this.r1 + this.r2 + d) / 2;
    alos = Math.sqrt(p * (p-this.r1) * (p-this.r2) * (p-d)) * 2;
    angc1 = 2 * Math.acos((-this.r2*this.r2 + this.r1*this.r1 + d*d)/(2*this.r1*d));
    ac1 = angc1 * this.r1*this.r1 / 2;
    angc2 = 2 * Math.acos((-this.r1*this.r1 + this.r2*this.r2 + d*d)/(2*this.r2*d));
    ac2 = angc2 * this.r2*this.r2 / 2;
    s = ac1 + ac2 - alos;
    
    if (this.tps < Math.PI)
        lum = this.l1 + this.l2 - (s / Math.PI) * Math.pow(this.temp2, 4);
    else
        lum = this.l1 + this.l2 - (s / Math.PI) * Math.pow(this.temp1, 4);
    lum = facteur * lum;
    return(lum);
}

Binaires.prototype.dessinImage = function() {
    var x1, y1, x2, y2, rapport, ir1, ix1, iy1, ir2, ix2, iy2;
    
    x1 = this.rorb1 * Math.cos(this.tps + Math.PI);
    y1 = this.rorb1 * Math.sin(this.inc) * Math.sin(this.tps + Math.PI);
    x2 = this.rorb2 * Math.cos(this.tps);
    y2 = this.rorb2 * Math.sin(this.inc) * Math.sin(this.tps);
    rapport = this.imgdimx / (2 * (this.rorb2 + this.r2*2));
    
    this.ctx.fillStyle = 'black';
    this.ctx.fillRect(0, 0, this.imgdimx-1, this.imgdimy-1);
    
    ir1 = Math.round(this.r1*rapport);
    ix1 = Math.round(this.imgdimx/2 + x1*rapport);
    iy1 = Math.round(this.imgdimy/2 - y1*rapport);
    ir2 = Math.round(this.r2*rapport);
    ix2 = Math.round(this.imgdimx/2 + x2*rapport);
    iy2 = Math.round(this.imgdimy/2 - y2*rapport);
    
    if (this.tps < Math.PI) {
        this.ctx.fillStyle = this.couleur2;
        this.ctx.beginPath();
        this.ctx.arc(ix2, iy2, ir2, 0, 2*Math.PI, false);
        this.ctx.fill();
    }
    
    this.ctx.fillStyle = this.couleur1;
    this.ctx.beginPath();
    this.ctx.arc(ix1, iy1, ir1, 0, 2*Math.PI, false);
    this.ctx.fill();
    
    if (this.tps >= Math.PI) {
        this.ctx.fillStyle = this.couleur2;
        this.ctx.beginPath();
        this.ctx.arc(ix2, iy2, ir2, 0, 2*Math.PI, false);
        this.ctx.fill();
    }
    return(this.canvas);
}

// tentative de représentation des couleurs apparentes
Binaires.prototype.couleurEtoile = function(temperature) {
    var r,v,b;
    
    if (temperature < 3000) {
        r = 255;
        v = 0;
        b = 0;
    } else if (temperature < 6000) {
        r = 255;
        v = (int)(255*(temperature-3000)/3000);
        b = 0;
    } else if (temperature < 10000) {
        b = Math.floor(255*(temperature-6000)/4000);
        r = 255;
        v = 255;
    } else if (temperature < 50000) {
        b = 255;
        r = 255 - Math.floor(255*(temperature-10000)/40000);
        v = 255 - Math.floor(255*(temperature-10000)/40000);
    } else {
        r = 0;
        v = 0;
        b = 255;
    }
    return('rgb(' + r + ',' + v + ',' + b + ')');
}

